/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "entity.h"
#include "assert/assert.h"
#include "input.h"
#include "display/render.h"
#include "world/world.h"
#include "physics/physics.h"

void playerInit() {
  entity_t *ent = &ENTITIES[0];

  entityInit(ent, ENTITY_TYPE_PLAYER);
  ent->id = PLAYER_ENTITY_ID;

  ent->x = WORLD_PLAYER_SPAWN_X;
  ent->y = WORLD_PLAYER_SPAWN_Y;
}

void playerNPCInit(entity_t *entity) {
  assertNotNull(entity, "Entity pointer cannot be NULL");
  assertTrue(entity->type == ENTITY_TYPE_PLAYER, "Entity type must be PLAYER");
}

void playerNPCUpdate(entity_t *entity) {
  assertNotNull(entity, "Entity pointer cannot be NULL");
  assertTrue(entity->type == ENTITY_TYPE_PLAYER, "Entity type must be PLAYER");

  if(inputIsDown(INPUT_BIND_UP)) {
    if(inputIsDown(INPUT_BIND_LEFT)) {
      entity->vx = -PLAYER_MOVE_SPEED_XY;
      entity->vy = -PLAYER_MOVE_SPEED_XY;

      if(entity->dir != ENTITY_DIR_NORTH && entity->dir != ENTITY_DIR_WEST) {
        entity->dir = ENTITY_DIR_NORTH;
      }
    } else if(inputIsDown(INPUT_BIND_RIGHT)) {
      entity->vx = PLAYER_MOVE_SPEED_XY;
      entity->vy = -PLAYER_MOVE_SPEED_XY;

      if(entity->dir != ENTITY_DIR_NORTH && entity->dir != ENTITY_DIR_EAST) {
        entity->dir = ENTITY_DIR_NORTH;
      }
    } else {
      entity->vy = -PLAYER_MOVE_SPEED;
      entity->vx = 0;
      entity->dir = ENTITY_DIR_NORTH;
    }
  } else if(inputIsDown(INPUT_BIND_DOWN)) {
    if(inputIsDown(INPUT_BIND_LEFT)) {
      entity->vx = -PLAYER_MOVE_SPEED_XY;
      entity->vy = PLAYER_MOVE_SPEED_XY;

      if(entity->dir != ENTITY_DIR_SOUTH && entity->dir != ENTITY_DIR_WEST) {
        entity->dir = ENTITY_DIR_SOUTH;
      }
    } else if(inputIsDown(INPUT_BIND_RIGHT)) {
      entity->vx = PLAYER_MOVE_SPEED_XY;
      entity->vy = PLAYER_MOVE_SPEED_XY;

      if(entity->dir != ENTITY_DIR_SOUTH && entity->dir != ENTITY_DIR_EAST) {
        entity->dir = ENTITY_DIR_SOUTH;
      }
    } else {
      entity->vy = PLAYER_MOVE_SPEED;
      entity->vx = 0;
      entity->dir = ENTITY_DIR_SOUTH;
    }
  } else if(inputIsDown(INPUT_BIND_LEFT)) {
    entity->vx = -PLAYER_MOVE_SPEED;
    entity->vy = 0;
    entity->dir = ENTITY_DIR_WEST;
  } else if(inputIsDown(INPUT_BIND_RIGHT)) {
    entity->vx = PLAYER_MOVE_SPEED;
    entity->vy = 0;
    entity->dir = ENTITY_DIR_EAST;
  } else {
    entity->vx = 0;
    entity->vy = 0;

    // Interact
    if(inputPressed(INPUT_BIND_ACTION)) {
      entity_t *other = ENTITIES;
      do {
        if(other == entity || other->type == ENTITY_TYPE_NULL) {
          other++;
          continue;
        }

        fixed248_t distanceX = fx248Subfx248(other->x, entity->x);
        fixed248_t distanceY = fx248Subfx248(other->y, entity->y);
        fixed248_t distance = fx248Sqrt(
          fx248Addfx248(
            fx248Mulfx248(distanceX, distanceX),
            fx248Mulfx248(distanceY, distanceY)
          )
        );
        
        if(other->id == 102) {
          printf("Distance to entity %u: %f\n", other->id, fx248Tof32(distance));
        }

        if(distance > PLAYER_INTERACT_RANGE) {
          other++;
          continue;
        }
        

        printf("Interacting with entity %u\n", other->id);
        other++;

      } while(other != ENTITIES + ENTITY_COUNT_MAX);
    }
  }
}